import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/objectWithoutPropertiesLoose";
import _assertThisInitialized from "@babel/runtime/helpers/assertThisInitialized";
import _inheritsLoose from "@babel/runtime/helpers/inheritsLoose";
import _extends from "@babel/runtime/helpers/extends";
import * as React from 'react';
import { VariableSizeList } from 'react-window';
import { Row } from './utils';

var computeTree = function computeTree(_temp, _ref, _ref2) {
  var _ref3 = _temp === void 0 ? {} : _temp,
      _ref3$refreshNodes = _ref3.refreshNodes,
      refreshNodes = _ref3$refreshNodes === void 0 ? false : _ref3$refreshNodes,
      _ref3$useDefaultHeigh = _ref3.useDefaultHeight,
      useDefaultHeight = _ref3$useDefaultHeigh === void 0 ? false : _ref3$useDefaultHeigh,
      _ref3$useDefaultOpenn = _ref3.useDefaultOpenness,
      useDefaultOpenness = _ref3$useDefaultOpenn === void 0 ? false : _ref3$useDefaultOpenn;

  var treeWalker = _ref.treeWalker;
  var recomputeTree = _ref2.recomputeTree,
      prevRecords = _ref2.records,
      resetAfterId = _ref2.resetAfterId;
  var order = [];

  var records = _extends({}, prevRecords);

  var iter = treeWalker(refreshNodes);

  if (useDefaultHeight || useDefaultOpenness) {
    for (var _id in records) {
      if (useDefaultHeight) {
        records[_id].height = records[_id].data.defaultHeight;
      }

      if (useDefaultOpenness) {
        records[_id].isOpen = records[_id].data.isOpenByDefault;
      }
    }
  }

  var isPreviousOpened = false;

  while (true) {
    var _iter$next = iter.next(isPreviousOpened),
        done = _iter$next.done,
        value = _iter$next.value;

    if (done || !value) {
      break;
    }

    var _id2 = void 0;

    if (typeof value === 'string' || typeof value === 'symbol') {
      _id2 = value;

      if (useDefaultOpenness) {
        records[_id2].isOpen = records[_id2].data.isOpenByDefault;
      }

      if (useDefaultHeight) {
        records[_id2].height = records[_id2].data.defaultHeight;
      }
    } else {
      (function () {
        _id2 = value.id;
        var defaultHeight = value.defaultHeight,
            isOpenByDefault = value.isOpenByDefault;
        var record = records[_id2];

        if (!record) {
          record = {
            data: value,
            height: defaultHeight,
            isOpen: isOpenByDefault,
            resize: function resize(height, shouldForceUpdate) {
              record.height = height;
              resetAfterId(record.data.id, shouldForceUpdate);
            },
            toggle: function toggle() {
              try {
                record.isOpen = !record.isOpen;
                return Promise.resolve(recomputeTree({
                  refreshNodes: record.isOpen,
                  useDefaultHeight: true
                })).then(function () {});
              } catch (e) {
                return Promise.reject(e);
              }
            }
          };
          records[_id2] = record;
        } else {
          record.data = value;

          if (useDefaultOpenness) {
            record.isOpen = isOpenByDefault;
          }

          if (useDefaultHeight) {
            record.height = defaultHeight;
          }
        }
      })();
    }

    order.push(_id2);
    isPreviousOpened = records[_id2].isOpen;
  }

  return {
    order: order,
    records: records
  };
};

var VariableSizeTree =
/*#__PURE__*/
function (_React$PureComponent) {
  _inheritsLoose(VariableSizeTree, _React$PureComponent);

  VariableSizeTree.getDerivedStateFromProps = function getDerivedStateFromProps(props, state) {
    var component = props.children,
        treeData = props.itemData,
        treeWalker = props.treeWalker;
    var oldTreeWalker = state.treeWalker,
        order = state.order;
    return _extends({
      component: component,
      treeData: treeData
    }, treeWalker !== oldTreeWalker || !order ? computeTree({
      refreshNodes: true
    }, props, state) : null);
  };

  function VariableSizeTree(props, context) {
    var _this;

    _this = _React$PureComponent.call(this, props, context) || this;
    _this.list = React.createRef();
    _this.getItemSize = _this.getItemSize.bind(_assertThisInitialized(_this));
    _this.state = {
      component: props.children,
      recomputeTree: _this.recomputeTree.bind(_assertThisInitialized(_this)),
      records: {},
      resetAfterId: _this.resetAfterId.bind(_assertThisInitialized(_this)),
      treeWalker: props.treeWalker
    };
    return _this;
  }

  var _proto = VariableSizeTree.prototype;

  _proto.recomputeTree = function recomputeTree(options) {
    try {
      var _this3 = this;

      return Promise.resolve(new Promise(function (resolve) {
        _this3.setState(function (prevState) {
          return computeTree(options, _this3.props, prevState);
        }, function () {
          if (options == null ? void 0 : options.useDefaultHeight) {
            var _this3$list$current;

            (_this3$list$current = _this3.list.current) == null ? void 0 : _this3$list$current.resetAfterIndex(0, true);
          }

          resolve();
        });
      }));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.resetAfterId = function resetAfterId(id, shouldForceUpdate) {
    var _this$list$current;

    if (shouldForceUpdate === void 0) {
      shouldForceUpdate = false;
    }

    (_this$list$current = this.list.current) == null ? void 0 : _this$list$current.resetAfterIndex(this.state.order.indexOf(id), shouldForceUpdate);
  };

  _proto.scrollTo = function scrollTo(scrollOffset) {
    var _this$list$current2;

    (_this$list$current2 = this.list.current) == null ? void 0 : _this$list$current2.scrollTo(scrollOffset);
  };

  _proto.scrollToItem = function scrollToItem(id, align) {
    var _this$list$current3;

    (_this$list$current3 = this.list.current) == null ? void 0 : _this$list$current3.scrollToItem(this.state.order.indexOf(id) || 0, align);
  };

  _proto.render = function render() {
    var _this$props = this.props,
        children = _this$props.children,
        itemSize = _this$props.itemSize,
        rowComponent = _this$props.rowComponent,
        treeWalker = _this$props.treeWalker,
        rest = _objectWithoutPropertiesLoose(_this$props, ["children", "itemSize", "rowComponent", "treeWalker"]);

    return React.createElement(VariableSizeList, Object.assign({}, rest, {
      itemData: this.state,
      itemCount: this.state.order.length // tslint:disable-next-line:no-unbound-method
      ,
      itemSize: itemSize || this.getItemSize,
      ref: this.list
    }), rowComponent);
  };

  _proto.getItemSize = function getItemSize(index) {
    var _this$state = this.state,
        order = _this$state.order,
        records = _this$state.records;
    return records[order[index]].height;
  };

  return VariableSizeTree;
}(React.PureComponent);

VariableSizeTree.defaultProps = {
  rowComponent: Row
};
export { VariableSizeTree as default };