import * as React from 'react';
import { FixedSizeList } from 'react-window';
import { Row } from './utils';

const computeTree = ({
  refreshNodes = false,
  useDefaultOpenness = false
} = {}, {
  treeWalker
}, {
  records: prevRecords,
  recomputeTree
}) => {
  const order = [];
  const records = { ...prevRecords
  };
  const iter = treeWalker(refreshNodes);

  if (useDefaultOpenness) {
    for (const id in records) {
      records[id].isOpen = records[id].data.isOpenByDefault;
    }
  }

  let isPreviousOpened = false;

  while (true) {
    const {
      done,
      value
    } = iter.next(isPreviousOpened);

    if (done || !value) {
      break;
    }

    let id;

    if (typeof value === 'string' || typeof value === 'symbol') {
      id = value;

      if (useDefaultOpenness) {
        records[id].isOpen = records[id].data.isOpenByDefault;
      }
    } else {
      ({
        id
      } = value);
      const {
        isOpenByDefault
      } = value;
      let record = records[id];

      if (!record) {
        record = {
          data: value,
          isOpen: isOpenByDefault,

          async toggle() {
            record.isOpen = !record.isOpen;
            await recomputeTree({
              refreshNodes: record.isOpen
            });
          }

        };
        records[id] = record;
      } else {
        record.data = value;

        if (useDefaultOpenness) {
          record.isOpen = isOpenByDefault;
        }
      }
    }

    order.push(id);
    isPreviousOpened = records[id].isOpen;
  }

  return {
    order,
    records
  };
};

export default class FixedSizeTree extends React.PureComponent {
  static getDerivedStateFromProps(props, state) {
    const {
      children: component,
      itemData: treeData,
      treeWalker
    } = props;
    const {
      treeWalker: oldTreeWalker,
      order
    } = state;
    return {
      component,
      treeData,
      ...(treeWalker !== oldTreeWalker || !order ? computeTree({
        refreshNodes: true
      }, props, state) : null)
    };
  }

  constructor(props, context) {
    super(props, context);
    this.list = React.createRef();
    this.state = {
      component: props.children,
      recomputeTree: this.recomputeTree.bind(this),
      records: {},
      treeWalker: props.treeWalker
    };
  }

  async recomputeTree(options) {
    return new Promise(resolve => {
      this.setState(prevState => computeTree(options, this.props, prevState), resolve);
    });
  }

  scrollTo(scrollOffset) {
    var _this$list$current;

    (_this$list$current = this.list.current) == null ? void 0 : _this$list$current.scrollTo(scrollOffset);
  }

  scrollToItem(id, align) {
    var _this$list$current2;

    (_this$list$current2 = this.list.current) == null ? void 0 : _this$list$current2.scrollToItem(this.state.order.indexOf(id) || 0, align);
  }

  render() {
    const {
      children,
      treeWalker,
      rowComponent,
      ...rest
    } = this.props;
    return React.createElement(FixedSizeList, Object.assign({}, rest, {
      itemData: this.state,
      itemCount: this.state.order.length,
      ref: this.list
    }), rowComponent);
  }

}
FixedSizeTree.defaultProps = {
  rowComponent: Row
};